# frozen_string_literal: true

module Hyrax
  module Crc1280Experiments
    class ComplexModalitiesController < ApplicationController
      include ComplexHelper
      before_action :set_crc1280_experiment, except: %i[validate_modality_title]
      before_action :set_complex_session, only: %i[new create validate_modality_title]
      before_action :set_complex_modality, only: %i[show edit update]
      before_action :check_tombstone, except: %i[validate_modality_title]
      before_action :authorize, only: [:new, :edit]
      with_themed_layout :decide_layout

      def new
        @complex_modality = ComplexModality.new
      end

      def edit
        @complex_session = @complex_modality.complex_session
      end

      def show
        @complex_session = @complex_modality.complex_session
        @modality_members = Kaminari.paginate_array(@complex_modality.file_sets).page(params[:page] || 1).per(params[:per_page] || 10)
        @workflow = WorkflowPresenter.new(@crc1280_experiment, current_ability)
      end

      def create
        @complex_modality = ComplexModality.new(complex_modality_params.merge!({ parent_source_identifier: @complex_session.source_identifier }))

        if @complex_modality.save
          update_file_set
          message = 'Complex modality was successfully created.'
          flash[:alert] = save_alert if params[:uploaded_files].present?
          redirect_to crc1280_experiment_complex_modality_path(@complex_modality.crc1280_experiment, @complex_modality), notice: message
        else
          render :new
        end
      end

      def update
        if @complex_modality.update(complex_modality_params)
          update_file_set
          message = 'Complex modality was successfully updated.'
          flash[:alert] = save_alert if params[:uploaded_files].present?
          redirect_to crc1280_experiment_complex_modality_path(@complex_modality.crc1280_experiment, @complex_modality), notice: message
        else
          render :edit
        end
      end

      def validate_modality_title
        folder_name = sanitize_title(params[:folder_name])
        modalities = @complex_session.complex_modalities
        .where(s3_folder_name: folder_name)
        .where.not(id: params[:current_object_id])

        invalid_length = false
        if folder_name.length < 3 or folder_name.length > 63
          invalid_length = true
          message = I18n.t('rdms.errors.complex_modality.title.length', value: folder_name)
        end
        message = I18n.t('rdms.errors.complex_modality.title.uniqueness', value: folder_name) if modalities.present?
        render json: { data_exist: (modalities.present? || invalid_length), error_message: message }
      end

      private

      def set_crc1280_experiment
        @crc1280_experiment = ::Hyrax.query_service.find_by(id: params[:crc1280_experiment_id])
      end

      def set_complex_session
        @complex_session = ComplexSession.find(params[:add_modalities_to_session] || params[:parent_id])
      end

      def set_complex_modality
        @complex_modality = ComplexModality.find(params[:id])
      end

      def complex_modality_params
        params.require(:complex_modality).permit(:modality_title, :modality)
      end

      def decide_layout
        layout = case action_name
                 when 'show'
                   '1_column'
                 else
                   'dashboard'
                 end
        File.join(theme, layout)
      end

      def update_file_set
        if params[:uploaded_files].present? and params[:uploaded_files].any?
          uploaded_files = Hyrax::UploadedFile.find(params[:uploaded_files])
          Hyrax::WorkUploadsHandler.new(work: @crc1280_experiment).add(files: uploaded_files, file_set_params: [{for_complex_identifier: @complex_modality.source_identifier, for_complex_type: 'ComplexModality'}]).attach

        end
      end

      def save_alert
        'Your files are being processed by RDMS in the background. The metadata and access controls you specified are being applied. You may need to refresh this page to see these updates.'
      end

      def check_tombstone
        if @crc1280_experiment.is_tombstoned == true
          redirect_to main_app.polymorphic_path(@crc1280_experiment)
        end
      end

      def authorize
        redirect_to main_app.polymorphic_path(@crc1280_experiment) unless current_user.can?(:edit, @crc1280_experiment)
      end
    end
  end
end
